import {NativeLibraryWrapperService} from '../native-library-wrapper.service';
import {PunycodeEncoder} from './punycode-encoder';
import createSpyObj = jasmine.createSpyObj;
import Spy = jasmine.Spy;

describe('PunycodeEncoder', () => {
  let sut: PunycodeEncoder;
  let nativeWrapperSpy: Partial<NativeLibraryWrapperService>;
  let encodeSpy: Spy;

  beforeEach(() => {
    nativeWrapperSpy = {punycode: createSpyObj(['encode'])};
    encodeSpy = nativeWrapperSpy.punycode.encode as Spy;
    sut = new PunycodeEncoder(nativeWrapperSpy as NativeLibraryWrapperService);
  });

  it('should create an instance', () => {
    expect(sut).toBeTruthy();
  });

  it('should have a display name', () => {
    expect(sut.getDisplayname()).toBeTruthy();
  });

  it('should have the id "encodepunycode"', () => {
    expect(sut.getId()).toEqual('encodepunycode');
  });

  it('should call through to the native punycode encoder', () => {
    const testInput = 'My input';
    const expectedOutput = 'It worked';
    encodeSpy.and.returnValue(expectedOutput);
    const result: string = sut.convert(testInput);
    expect(result).toEqual(expectedOutput);
    expect(encodeSpy).toHaveBeenCalledWith(testInput);
  });
});
